@@ -8,6 +8,7 @@ use Test::Run::Output;
 use Test::Run::Base;
 use Test::Run::Assert;
 use Test::Run::Obj::Structs;
+use Test::Run::Obj::Error;
 use Exporter;
 use Benchmark;
 use Config;
@@ -35,11 +36,11 @@ Test::Run - Run Perl standard test scripts with statistics
 
 =head1 VERSION
 
-Version 2.53_02
+Version 0.0100_06
 
 =cut
 
-$VERSION = "0.0100_05";
+$VERSION = "0.0100_06";
 
 # Backwards compatibility for exportable variable names.
 # REMOVED *verbose  = *Verbose;
@@ -282,14 +283,12 @@ one of the messages in the DIAGNOSTICS section.
 
 =cut
 
-sub runtests {
+sub _real_runtests
+{
     my $self = shift;
-    my (%args) = (@_);
-
-    local ($\, $,);
-
     my($failedtests) =
         $self->_run_all_tests();
+
     $self->_show_results();
 
     my $ok = $self->_all_ok();
@@ -300,6 +299,42 @@ sub runtests {
     return $ok;
 }
 
+sub _handle_runtests_error
+{
+    my $self = shift;
+    my (%args) = @_;
+    my $error = $args{'error'};
+
+    if (UNIVERSAL::isa($error, "Test::Run::Obj::Error::TestsFail"))
+    {
+        die ($@->text() . "\n");
+    }
+    else
+    {
+        die $@;
+    }
+}
+
+sub runtests
+{
+    my $self = shift;
+
+    local ($\, $,);
+
+    my $ok = eval { $self->_real_runtests(@_) };
+    if ($@)
+    {
+        return $self->_handle_runtests_error(
+            'ok' => $ok, 
+            'error' => $@,
+        );
+    }
+    else
+    {
+        return $ok;
+    }
+}
+
 =begin _private
 
 =item B<_all_ok>
@@ -764,23 +799,66 @@ sub _leader_width {
     my $tests = $self->test_files();
 
     my $maxlen = 
-        $self->__max_num(map {length($_)} @$tests);
+        $self->__max_num_flat(map {length($_)} @$tests);
     my $maxsuflen =
-        $self->__max_num(map {length(/\.(\w+)$/ ? $1 : '')} @$tests);
+        $self->__max_num_flat(map {length(/\.(\w+)$/ ? $1 : '')} @$tests);
 
     # + 3 : we want three dots between the test name and the "ok"
     return $maxlen + 3 - $maxsuflen;
 }
 
+
+=head2 $self->_report('channel' => $channel, 'msg' => $message);
+
+Reports the C<$message> message to channel C<$channel>. This can be overrided
+by derived classes to do alternate functionality besides calling 
+_print_message(), also different based on the channel.
+Currently available channels are:
+
+=over 4
+
+=item 'success'
+
+The success report.
+
+=back
+
+=cut
+
+sub _report
+{
+    my $self = shift;
+    my (%args) = @_;
+    my $msg = $args{'msg'};
+    return $self->_print_message($msg);    
+}
+
+sub _get_success_msg
+{
+    my $self = shift;
+    return "All tests successful" . $self->_get_bonusmsg() . ".";
+}
+
 sub _report_success
 {
     my $self = shift;
-    $self->_print_message("All tests successful" . $self->_get_bonusmsg() . ".");
+    $self->_report(
+        'channel' => "success",
+        'msg' => $self->_get_success_msg(),
+    );
+}
+
+sub _get_fail_no_tests_run_text
+{
+    return "FAILED--no tests were run for some reason."
 }
 
 sub _fail_no_tests_run
 {
-    die "FAILED--no tests were run for some reason.\n";
+    my $self = shift;
+    die Test::Run::Obj::Error::TestsFail->new(
+        text => $self->_get_fail_no_tests_run_text(),
+    );
 }
 
 sub _fail_no_tests_output
@@ -840,6 +918,13 @@ sub _get_format_failed_str_len
     return length($self->_get_format_failed_str());
 }
 
+sub __max_num_flat
+{
+    my $self = shift;
+    my $n = shift;
+    return $self->__max_num($n, [@_]);
+}
+
 sub __max_num
 {
     my ($self, $max, $others )= @_;
@@ -1036,7 +1121,8 @@ sub _show_results {
     {
         $self->_fail_no_tests_output();
     }
-    else {
+    else
+    {
         $self->_fail_other();
     }
     $self->_print_final_stats();
@@ -1119,13 +1205,15 @@ sub _get_skipped_bonusmsg
     my $sub_skipped = $tot->sub_skipped();
     my $skipped = $tot->skipped();
 
+    # TODO: Refactor it.
     my $sub_skipped_msg =
         "$sub_skipped subtest" . $self->_get_s($sub_skipped);
 
+    my $comma = ", ";
     if ($skipped)
     {
         return 
-            ", $skipped test" .
+            $comma . "$skipped test" .
             $self->_get_s($skipped) .
             ($sub_skipped ? (" and " . $sub_skipped_msg) : "") .
             ' skipped'
@@ -1133,7 +1221,12 @@ sub _get_skipped_bonusmsg
     }
     elsif ($sub_skipped)
     {
-        return "$sub_skipped_msg skipped";
+        # Should be a comma here too.
+        return $comma . "$sub_skipped_msg skipped";
+    }
+    else
+    {
+        return "";
     }
 }
 
